/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of the advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File              Optimizations.c
* Owner             b05111
* Version           0.0
* Date              Nov-08-2014
* Classification    General Business Information
********************************************************************************
* Detailed Description:
********************************************************************************
Revision History:
Version  Date         Author        Description of Changes
0.0      Nov-08-2014  b05111        Initial version
*******************************************************************************/

#include "Optimizations.h"
#include "MPC5674F_MVxA264.h"

/*******************************************************************************
* Local function prototypes
*******************************************************************************/

static void FlashConfig(void);
static asm void BranchPredictionEnable(void);
static asm void ICache_init(void);
static asm void DCache_init(void);
static asm void Enable_DCache_Copyback(void);
static asm void MMUFlashIsCacheInhibited(void);
static asm void MMUFlashIsCacheable(void);
static asm void MMUSRAMIsCacheable(void);


/*******************************************************************************
* Local functions
*******************************************************************************/

/*******************************************************************************
Function Name : FlashConfig
Engineer      : b08110
Date          : Mar-11-2011
Parameters    : 
Modifies      : 
Returns       : 
Notes         : optimize read flash wait states to the fsys
                - the code that writes to the RWSC must not execute from flash
                - thus we execute from stack (= RAM)
Issues        : it could be modified for other frequecies as well
*******************************************************************************/
static void FlashConfig(void) 
{    
    unsigned int mem_write_code [] = {

		#if __option(vle)
			/*for processors which support VLE only or for 'VLE on' option*/
    		0xD0344400, /* stw r3,(0)r4 machine code: writes r3 contents to addr in r4 then se_nop*/
    		0x7C0006AC, /* mbar machine code: ensure prior store completed */
    		0x44000004  /* blr machine code: branches to return address in link register */
		#else
   			0x90640000, /* stw r3,(0)r4 machine code: writes r3 contents to addr in r4 */
   			0x7C0006AC, /* mbar machine code: ensure prior store completed */
   			0x4E800020  /* blr machine code: branches to return address in link register */
		#endif // __option(vle)

	};
	
    typedef void (*mem_write_code_ptr_t)(unsigned int, unsigned int);

    (*((mem_write_code_ptr_t)mem_write_code)) 	/* cast mem_write_code as func ptr*/
                                 /* * de-references func ptr, i.e. converts to func*/
     (FLASH_BIUCR_264,            /* which passes integer (in r3) */
      (unsigned int)&FLASH_A.BIUCR.R);
}

/*******************************************************************************
Function Name : BranchPredictionEnable
Engineer      : b08110
Date          : Feb-22-2010
Parameters    : 
Modifies      : r3, bucsr
Returns       : 
Notes         : turn on branch prediction and invalidate branch target buffer
Issues        : 
*******************************************************************************/
static asm void BranchPredictionEnable(void)
{
    nofralloc
    
    /* turn on branch prediction and invalidate branch target buffer */
    lis   r3, 0x0
    ori   r3, r3, 0x0201
    mtbucsr r3

    blr   
}


/*******************************************************************************
Function Name : MMUFlashIsCacheInhibited
Engineer      : b08110
Date          : Feb-25-2010
Parameters    : 
Modifies      : r3, TLB1
Returns       : 
Notes         : modify TLB1 (by BAM or degugger script) 
                  to be cache inhibited. TLB1 is Internal Flash.                   
Issues        : 
*******************************************************************************/
static asm void MMUFlashIsCacheInhibited(void)
{
    nofralloc
    
    lis    r3, 0x1001    /* Select TLB entry #, define R/W replacment control */
    mtMAS0 r3            /* Load MAS0 with 0x1001 0000 for TLB entry #1 */

    tlbre                /* Get TLB entry # information */
  
    mfspr  r3, MAS2
    ori	 r3, r3, 0x0008  /* set I bit (Page is cache inhibited) */
	                      
    mtMAS2 r3
  
    msync        /* make sure we finished all memory accesses */
  
    tlbwe        /* Write entry defined in MAS0 (entry 1 here) to MMU TLB */
  
    isync        /* Wait for tlbwe to complete, then flush instruction buffer */
    
    blr
}


/*******************************************************************************
Function Name : MMUFlashIsCacheable
Engineer      : b08110
Date          : Feb-25-2010
Parameters    : 
Modifies      : r3, TLB1
Returns       : 
Notes         : modify TLB1 to be cacheable. TLB1 is Internal Flash.                   
Issues        : 
*******************************************************************************/
static asm void MMUFlashIsCacheable(void)
{
    nofralloc
    
    lis    r3, 0x1001     /* Select TLB entry #, define R/W replacment control */
    mtMAS0 r3             /* Load MAS0 with 0x1001 0000 for TLB entry #1 */

    tlbre                 /* Get TLB entry # information */
  
    mfspr   r3, MAS2
    lis     r4, 0xFFFF
    ori     r4, r4, 0xFFF7    
    and     r3, r3, r4    /* clear I bit (cacheable) */
	                      
    mtMAS2 r3
  
    msync        /* make sure we finished all memory accesses */
  
    tlbwe        /* Write entry defined in MAS0 (entry 1 here) to MMU TLB */
  
    isync        /* Wait for tlbwe to complete, then flush instruction buffer */
    blr    
}


/*******************************************************************************
Function Name : MMUSRAMIsCacheable
Engineer      : b05111
Date          : Oct-19-2012
Parameters    : 
Modifies      : r3, TLB3
Returns       : 
Notes         : modify TLB3 to be cacheable. TLB3 is internal SRAM
Issues        : 
*******************************************************************************/
static asm void MMUSRAMIsCacheable(void)
{
    nofralloc

#ifdef DCACHE_COPYBACK_MODE
    /* copy-back */    
    lis r3, 0x1003
    mtMAS0 r3

    lis r3,0xC000
    ori r3, r3, 0x0400
    mtMAS1 r3

    lis r3, 0x4000
    ori r3, r3, 0x0026
    mtMAS2 r3

    lis r3, 0x4000
    ori r3, r3, 0x003F
    mtMAS3 r3
    msync
    tlbwe
    isync

#else

    /* write-through */
    lis r3, 0x1003
    mtMAS0 r3

    lis r3,0xC000
    ori r3, r3, 0x0400
    mtMAS1 r3

    lis r3, 0x4000
    ori r3, r3, 0x0034
    mtMAS2 r3

    lis r3, 0x4000
    ori r3, r3, 0x003F
    mtMAS3 r3
    msync
    tlbwe
    isync


#endif    
    
    blr    
}


/*******************************************************************************
Function Name : ICache_init
Engineer      : b05111
Date          : Oct-19-2012
Parameters    : 
Modifies      : 
Returns       : 
Notes         : Invalidate and Enable
Issues        : 
*******************************************************************************/
static asm void ICache_init(void)
{
    nofralloc
    
/**************************************************************************
# FUNCTION     : cfg_CACHE                                               *
# PURPOSE      : This function initializes the CACHE, using L1CSR1       *
#                register.                                                *
#                SEQUENCE:                                                *
#                - Start Cache Invalidate                                *
#                - Wait for invalidation to complete, check for Abort     *
#                  operation and restart invalidation if detected         *
#                - Enable Cache by setting CE bit                       *
#**************************************************************************/
cfg_CACHE:

    /*--------------------------------------------#
    # Invalidate - Set CINV                      #
    # bit in L1CSR1 Register                     #
    #--------------------------------------------*/
    lis   r5, 0x0000
    ori   r5, r5, 0x0002
    msync
    isync
    mtspr SPR_L1CSR1,r5

    /*-------------------------------------------#
    # Mask out CINV and CABT to see if           #
    # invalidation is complete (i.e. CINV=0,     #
    # CABT=0)                                    #
    #-------------------------------------------*/
label_CINV_check:
    /*-------------------------------------------#
    # Load Registers with Masks:                 #
    # Load CINV mask into R8                     #
    # Load CABT mask into R7                     #
    # Load CABT clear mask into R11              #
    #-------------------------------------------*/
    lis   r8, 0x0000
    ori   r8, r8, 0x0002
    lis   r7, 0x0000
    ori   r7, r7, 0x0004
    lis   r11, 0xFFFF
    ori   r11, r11, 0xFFFB
CHECK_CINV:

    /*-------------------------------------------#
    # Read SPR_L1CSR1 register, store in r9      #
    #-------------------------------------------*/
    mfspr r9, SPR_L1CSR1
    /*-------------------------------------------#
    # check for an ABORT of the cache invalidate #
    # operation                                  #
    #-------------------------------------------*/
    and.  r10, r7, r9
    beq   NO_ABORT
    /*-------------------------------------------#
    # If abort detected, clear CABT bit and      #
    # re-run invalidation                        #
    #-------------------------------------------*/
    and.  r10, r11, r9
    msync
    isync
    mtspr SPR_L1CSR1, r10
    b     cfg_CACHE

NO_ABORT:
    /*-------------------------------------------#
    # Check that invalidation has completed -    #
    # (CINV=0). Branch if invalidation not       #
    # complete.                                  #
    #-------------------------------------------*/
    and.  r10, r8, r9
    bne CHECK_CINV

    /*-------------------------------------------#
    # Enable cache by performing a               #
    # read/modify/write of the CE bit in the     #
    # SPR_L1CSR1 register                        #
    #-------------------------------------------*/
    mfspr r5, SPR_L1CSR1
    oris  r5, r5, 0x0000
    ori   r5, r5, 0x0001    /* Store SPR_L1CSR1 value to R5 (CE=1) */
    msync
    isync
    mtspr SPR_L1CSR1, r5        /* Write R5 to SPR_L1CSR1 register */

    blr
/**************************************************************************
# END OF cfg_CACHE                                                        *
#**************************************************************************/
}



/*******************************************************************************
Function Name : DCache_init
Engineer      : b05111
Date          : Oct-19-2012
Parameters    : 
Modifies      : 
Returns       : 
Notes         : Invalidate and Enable
Issues        : 
*******************************************************************************/
static asm void DCache_init(void)
{
    nofralloc
    
/**************************************************************************
# FUNCTION     : cfg_CACHE                                                *
# PURPOSE      : This function initializes the CACHE, using L1CSR1        *
#                register.                                                *
#                SEQUENCE:                                                *
#                - Start Cache Invalidate                                 *
#                - Wait for invalidation to complete, check for Abort     *
#                  operation and restart invalidation if detected         *
#                - Enable Cache by setting CE bit                         *
#**************************************************************************/
cfg_DCACHE:

    /*--------------------------------------------#
    # Invalidate - Set CINV                      #
    # bit in L1CSR0 Register                     #
    #--------------------------------------------*/
    lis   r5, 0x0000
    ori   r5, r5, 0x0002
    msync
    isync
    mtspr SPR_L1CSR0,r5

    /*-------------------------------------------#
    # Mask out CINV and CABT to see if           #
    # invalidation is complete (i.e. CINV=0,     #
    # CABT=0)                                    #
    #-------------------------------------------*/
label_DCINV_check:
    /*-------------------------------------------#
    # Load Registers with Masks:                 #
    # Load CINV mask into R8                     #
    # Load CABT mask into R7                     #
    # Load CABT clear mask into R11              #
    #-------------------------------------------*/
    lis   r8, 0x0000
    ori   r8, r8, 0x0002
    lis   r7, 0x0000
    ori   r7, r7, 0x0004
    lis   r11, 0xFFFF
    ori   r11, r11, 0xFFFB
CHECK_DCINV:

    /*-------------------------------------------#
    # Read SPR_L1CSR0 register, store in r9      #
    #-------------------------------------------*/
    mfspr r9, SPR_L1CSR0
    /*-------------------------------------------#
    # check for an ABORT of the cache invalidate #
    # operation                                  #
    #-------------------------------------------*/
    and.  r10, r7, r9
    beq   D_NO_ABORT
    /*-------------------------------------------#
    # If abort detected, clear CABT bit and      #
    # re-run invalidation                        #
    #-------------------------------------------*/
    and.  r10, r11, r9
    msync
    isync
    mtspr SPR_L1CSR0, r10
    b     cfg_DCACHE

D_NO_ABORT:
    /*-------------------------------------------#
    # Check that invalidation has completed -    #
    # (CINV=0). Branch if invalidation not       #
    # complete.                                  #
    #-------------------------------------------*/
    and.  r10, r8, r9
    bne CHECK_DCINV

    /*-------------------------------------------#
    # Enable cache by performing a               #
    # read/modify/write of the CE bit in the     #
    # SPR_L1CSR0 register                        #
    #-------------------------------------------*/
    mfspr r5, SPR_L1CSR0
    oris  r5, r5, 0x0000
    ori   r5, r5, 0x0001    /* Store SPR_L1CSR0 value to R5 (CE=1) */
    msync
    isync
    mtspr SPR_L1CSR0, r5        /* Write R5 to SPR_L1CSR0 register */

    blr
/**************************************************************************
# END OF cfg_CACHE                                                        *
#**************************************************************************/
}


/*******************************************************************************
Function Name : Enable_DCache_Copyback
Engineer      : b05111
Date          : Oct-19-2012
Parameters    : 
Modifies      : 
Returns       : 
Notes         : Invalidate and Enable
Issues        : 
*******************************************************************************/
#ifdef DCACHE_COPYBACK_MODE
static asm void Enable_DCache_Copyback(void)
{
    /*-------------------------------------------#
    # Enable cache by performing a               #
    # read/modify/write of the DCWM bit in the   #
    # SPR_L1CSR0 register                        #
    #-------------------------------------------*/
    mfspr r5, SPR_L1CSR0
    oris  r5, r5, 0x0010
    ori   r5, r5, 0x0000    /* Store SPR_L1CSR0 value to R5 (DCWM=1) */
    msync
    isync
    mtspr SPR_L1CSR0, r5        /* Write R5 to SPR_L1CSR0 register */

    blr
}
#endif


/*******************************************************************************
* Global functions
*******************************************************************************/

/*******************************************************************************
Function Name : Optimizations
Engineer      : b05111
Date          : Oct-19-2012
Parameters    : 
Modifies      : 
Returns       : 
Notes         : 
Issues        : 
*******************************************************************************/
void Optimizations(void)
{
    FlashConfig();
    ICache_init();
    DCache_init();
    #ifdef DCACHE_COPYBACK_MODE
        Enable_DCache_Copyback();
    #endif
    MMUFlashIsCacheable();
    BranchPredictionEnable();
    MMUSRAMIsCacheable();
}

