/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2004-2005 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. FREESCALE neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED TO THE PROJECT
BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE SERVICES. IN NO EVENT
SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF 
THIS AGREEMENT.

CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands or 
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of the advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File             eQADC_macros_MAMBA.h
* Owner            b05111
* Version          0.0   
* Date             Jun-18-2015
* Classification   General Business Information
* Brief            Conversion and Configuration Commands for eQADC
********************************************************************************
* Detailed Description:
********************************************************************************
Revision History:
Version  Date         Author    Description of Changes
0.0      Jun-18-2015  b05111    Initial version based on eQADC_macros_MONACO
*******************************************************************************/

#ifndef _EQADC_MACROS_H
#define _EQADC_MACROS_H

/*******************************************************************************
* Constants
*******************************************************************************/

/*******************************************************************************
* Macros 
*******************************************************************************/


/*******************************************************************************
*******************             PUSH/POP macros            *********************
*******************************************************************************/

#define PUSH_TO_CFIFO0_A(n)  EQADC_A.CFPR[0].R = n
#define PUSH_TO_CFIFO1_A(n)  EQADC_A.CFPR[1].R = n
#define PUSH_TO_CFIFO2_A(n)  EQADC_A.CFPR[2].R = n
#define PUSH_TO_CFIFO3_A(n)  EQADC_A.CFPR[3].R = n
#define PUSH_TO_CFIFO4_A(n)  EQADC_A.CFPR[4].R = n
#define PUSH_TO_CFIFO5_A(n)  EQADC_A.CFPR[5].R = n

#define POP_FROM_RFIFO0_A  EQADC_A.RFPR[0].R
#define POP_FROM_RFIFO1_A  EQADC_A.RFPR[1].R
#define POP_FROM_RFIFO2_A  EQADC_A.RFPR[2].R
#define POP_FROM_RFIFO3_A  EQADC_A.RFPR[3].R
#define POP_FROM_RFIFO4_A  EQADC_A.RFPR[4].R
#define POP_FROM_RFIFO5_A  EQADC_A.RFPR[5].R

#define PUSH_TO_CFIFO0_B(n)  EQADC_B.CFPR[0].R = n
#define PUSH_TO_CFIFO1_B(n)  EQADC_B.CFPR[1].R = n
#define PUSH_TO_CFIFO2_B(n)  EQADC_B.CFPR[2].R = n
#define PUSH_TO_CFIFO3_B(n)  EQADC_B.CFPR[3].R = n
#define PUSH_TO_CFIFO4_B(n)  EQADC_B.CFPR[4].R = n
#define PUSH_TO_CFIFO5_B(n)  EQADC_B.CFPR[5].R = n

#define POP_FROM_RFIFO0_B  EQADC_B.RFPR[0].R
#define POP_FROM_RFIFO1_B  EQADC_B.RFPR[1].R
#define POP_FROM_RFIFO2_B  EQADC_B.RFPR[2].R
#define POP_FROM_RFIFO3_B  EQADC_B.RFPR[3].R
#define POP_FROM_RFIFO4_B  EQADC_B.RFPR[4].R
#define POP_FROM_RFIFO5_B  EQADC_B.RFPR[5].R

/*******************************************************************************
*******************          CFIFO operation modes         *********************
*******************************************************************************/

// for register EQADC_CFCRx
// MODEx[0:3]  CFIFO Operation Mode
#define MODEx(x) (((x)&0xF)<<4)
  #define DISABLED                                                 (0)
  #define SOFTWARE_TRIGGER_SINGLE_SCAN                             (1)
  #define LOW_LEVEL_GATED_EXTERNAL_TRIGGER_SINGLE_SCAN             (2)
  #define HIGH_LEVEL_GATED_EXTERNAL_TRIGGER_SINGLE_SCAN            (3)
  #define FALLING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN                (4)
  #define RISING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN                 (5)
  #define FALLING_OR_RISING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN      (6)
  #define SOFTWARE_TRIGGER_CONTINUOUS_SCAN                         (9)
  #define LOW_LEVEL_GATED_EXTERNAL_TRIGGER_CONTINUOUS_SCAN         (10)
  #define HIGH_LEVEL_GATED_EXTERNAL_TRIGGER_CONTINUOUS_SCAN        (11)
  #define FALLING_EDGE_EXTERNAL_TRIGGER_CONTINUOUS_SCAN            (12)
  #define RISING_EDGE_EXTERNAL_TRIGGER_CONTINUOUS_SCAN             (13)
  #define FALLING_OR_RISING_EDGE_EXTERNAL_TRIGGER_CONTINUOUS_SCAN  (14)

// AMODE0[0:3]  CFIFO0 Advance Trigger Operation Mode
#define AMODE0(x) ((x)&0xF)
  #define DISABLED                                                 (0)
  #define SOFTWARE_TRIGGER_SINGLE_SCAN                             (1)
  #define LOW_LEVEL_GATED_EXTERNAL_TRIGGER_SINGLE_SCAN             (2)
  #define HIGH_LEVEL_GATED_EXTERNAL_TRIGGER_SINGLE_SCAN            (3)
  #define FALLING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN                (4)
  #define RISING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN                 (5)
  #define FALLING_OR_RISING_EDGE_EXTERNAL_TRIGGER_SINGLE_SCAN      (6)

/*******************************************************************************
*******************             COMMANDS FORMAT            *********************
*******************************************************************************/

//   Conversion Command Format for the Standard Configuration

#define EOQ            0x80000000               // End of Queue
#define PAUSE          0x40000000               // Pause
#define REP            0x20000000               // Repeat
#define BN             0x02000000               // Buffer Number
  #define B0             0x00000000               // Buffer 0
  #define B1             0x02000000               // Buffer 1
#define CAL            0x01000000               // Calibration Enable
#define MESSAGE_TAG(x) (((x)&0xF)<<20)          
  #define RFIFO0  0
  #define RFIFO1  1
  #define RFIFO2  2
  #define RFIFO3  3
  #define RFIFO4  4
  #define RFIFO5  5
  #define NULL_M  8  
#define LST(x)         (((x)&0x3)<<18)
  #define CYCLES_2    0
  #define CYCLES_8    1
  #define CYCLES_64   2
  #define CYCLES_128  3
#define TSR            0x00020000
#define FMT            0x00010000
#define CHANNEL(x)     (((x)&0xFF)<<8)


//   Conversion Command Format for Alternate Configurations

#define EOQ                0x80000000
#define PAUSE              0x40000000
#define REP                0x20000000
#define BN                 0x02000000
#define CAL                0x01000000
#define MESSAGE_TAG(x)     (((x)&0xF)<<20)
  #define RFIFO0  0
  #define RFIFO1  1
  #define RFIFO2  2
  #define RFIFO3  3
  #define RFIFO4  4
  #define RFIFO5  5
  #define NULL_M  8  
#define LST(x)             (((x)&0x3)<<18)
  #define CYCLES_2    0
  #define CYCLES_8    1
  #define CYCLES_64   2
  #define CYCLES_128  3
#define TSR                0x00020000
#define FFMT               0x00010000
#define CHANNEL(x)         (((x)&0xFF)<<8)
#define ALT_CONFIG_SEL(x)  (((x)&0xFF)<<0)
  #define ALT_CONF_1  0x08
  #define ALT_CONF_2  0x09
  #define ALT_CONF_3  0x0A
  #define ALT_CONF_4  0x0B
  #define ALT_CONF_5  0x0C
  #define ALT_CONF_6  0x0D
  #define ALT_CONF_7  0x0E
  #define ALT_CONF_8  0x0F


//   Write Configuration Command Message Format for On-Chip ADC Operation

#define EOQ              0x80000000
#define PAUSE            0x40000000
#define REP              0x20000000
#define BN               0x02000000
#define WRITE            0x00000000
#define ADC_REG_VALUE(x) (((x)&0xFFFF)<<8)
#define ADC_REG_ADDR(x)  (((x)&0xFF))
  #define conv_command_std   0x00
  #define ADC0_CR            0x01
  #define ADC1_CR            0x01
  #define ADC_TSCR           0x02
  #define ADC_TBCR           0x03
  #define ADC0_GCCR          0x04
  #define ADC1_GCCR          0x04
  #define ADC0_OCCR          0x05
  #define ADC1_OCCR          0x05
  #define conv_command_alt1  0x08
  #define conv_command_alt2  0x09
  #define conv_command_alt3  0x0A
  #define conv_command_alt4  0x0B
  #define conv_command_alt5  0x0C
  #define conv_command_alt6  0x0D
  #define conv_command_alt7  0x0E
  #define conv_command_alt8  0x0F
  #define ADC_ACR1           0x30
  #define ADC0_AGR1          0x31
  #define ADC1_AGR1          0x31
  #define ADC0_AOR1          0x32
  #define ADC1_AOR1          0x32
  #define ADC_ACR2           0x34
  #define ADC0_AGR2          0x35
  #define ADC1_AGR2          0x35
  #define ADC0_AOR2          0x36
  #define ADC1_AOR2          0x36
  #define ADC_ACR3           0x38
  #define ADC_ACR4           0x3C
  #define ADC_ACR5           0x40
  #define ADC_ACR6           0x44
  #define ADC_ACR7           0x48
  #define ADC_ACR8           0x4C
  #define ADC_PUDCR0         0x70
  #define ADC_PUDCR1         0x71
  #define ADC_PUDCR2         0x72
  #define ADC_PUDCR3         0x73
  #define ADC_PUDCR4         0x74
  #define ADC_PUDCR5         0x75
  #define ADC_PUDCR6         0x76
  #define ADC_PUDCR7         0x77
  #define ADC_REDTBR1        0xA0
  #define ADC_REDTBR2        0xA1


//   Read Configuration Command Message Format for On-Chip ADC Operation

#define EOQ             0x80000000
#define PAUSE           0x40000000
#define REP             0x20000000
#define BN              0x02000000
#define READ            0x01000000
#define MESSAGE_TAG(x)  (((x)&0xF)<<20)
  #define RFIFO0  0
  #define RFIFO1  1
  #define RFIFO2  2
  #define RFIFO3  3
  #define RFIFO4  4
  #define RFIFO5  5
  #define NULL_M  8  
#define ADC_REG_ADDR(x) (((x)&0xFF))
  #define conv_command_std   0x00
  #define ADC0_CR            0x01
  #define ADC1_CR            0x01
  #define ADC_TSCR           0x02
  #define ADC_TBCR           0x03
  #define ADC0_GCCR          0x04
  #define ADC1_GCCR          0x04
  #define ADC0_OCCR          0x05
  #define ADC1_OCCR          0x05
  #define conv_command_alt1  0x08
  #define conv_command_alt2  0x09
  #define conv_command_alt3  0x0A
  #define conv_command_alt4  0x0B
  #define conv_command_alt5  0x0C
  #define conv_command_alt6  0x0D
  #define conv_command_alt7  0x0E
  #define conv_command_alt8  0x0F
  #define ADC_ACR1           0x30
  #define ADC0_AGR1          0x31
  #define ADC1_AGR1          0x31
  #define ADC0_AOR1          0x32
  #define ADC1_AOR1          0x32
  #define ADC_ACR2           0x34
  #define ADC0_AGR2          0x35
  #define ADC1_AGR2          0x35
  #define ADC0_AOR2          0x36
  #define ADC1_AOR2          0x36
  #define ADC_ACR3           0x38
  #define ADC_ACR4           0x3C
  #define ADC_ACR5           0x40
  #define ADC_ACR6           0x44
  #define ADC_ACR7           0x48
  #define ADC_ACR8           0x4C
  #define ADC_PUDCR0         0x70
  #define ADC_PUDCR1         0x71
  #define ADC_PUDCR2         0x72
  #define ADC_PUDCR3         0x73
  #define ADC_PUDCR4         0x74
  #define ADC_PUDCR5         0x75
  #define ADC_PUDCR6         0x76
  #define ADC_PUDCR7         0x77
  #define ADC_REDTBR1        0xA0
  #define ADC_REDTBR2        0xA1

/*******************************************************************************
* Types
*******************************************************************************/

/*******************************************************************************
* Global variables
*******************************************************************************/

/*******************************************************************************
* Global functions
*******************************************************************************/

/*******************************************************************************
* Inline functions
*******************************************************************************/

#endif _EQADC_MACROS_H
